/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef PAPYRUSPOLYLINE_H
#define PAPYRUSPOLYLINE_H

#include <vector>

#include <papyrus/shape.h>
#include <papyrus/marker.h>

/**
 * \page guide_polyline Polylines
 * A polyline is drawn as a set of ordered vertices relative to the shape's x,y
 * coordinate. Polylines may be filled or unfilled. An unfilled polyline is more
 * conceptually familiar with what is implied by a polyline. A filled polyline
 * is essentially an unclosed polygon.
 *
 * The following contains an example that will add an unfilled polygon to a canvas.
 * The blue lines represent the x and y axes, and show through since an alpha value
 * of 0.9 is used.
 *
 * \image html polyline.png "Unfilled Polyline drawn by example code below"
 *
 * Note that \c set_outline() is used to set the line color.
 *
 * \dontinclude shapes.cpp
 * \skip add_polyline_unfilled
 * \until }
 *
 * The following illustrates the effects of filling a polyline.
 *
 * \image html polyline_filled.png "Filled Polyline drawn by example code below"
 *
 * \dontinclude shapes.cpp
 * \skip add_polyline_filled
 * \until }
 *
 */


namespace Papyrus {

class Vertex {
	public:
		Vertex(double vx=0.0, double vy=0.0, bool vrelative=false):
      x(vx), y(vy), relative(vrelative)
    {
		}

		virtual ~Vertex() { }

		double x, y;
    bool relative;
};

typedef std::vector<Vertex> Vertices;

  /**
   * A polyline is drawn as a set of ordered vertices relative to the shape's x,y
   * coordinate. Polylines may be filled or unfilled. An unfilled polyline is more
   * conceptually familiar with what is implied by a polyline. A filled polyline
   * is essentially an unclosed polygon.
   *
   * \image html polyline.png "Unfilled Polyline"
   * \image html polyline_filled.png "Filled Polyline"
   *
   * @author Rick L Vinyard Jr
   * TODO change marker handling to allow setting positional markers
   */
  class Polyline : public Shape {
    public:
      typedef PapyrusSmartPointer<Polyline> pointer;

      Polyline( Vertices vertices = Vertices() );

      /** Creates a polyline from a specified set of vertices */
      static Polyline::pointer create( Vertices vertices = Vertices() );

      ~Polyline();

      Polyline& operator=(const Polyline& other);

      LineStyle& line_style();

			Vertex get_vertex(unsigned n);

			const Vertices& get_vertices();

			void add_vertex(Vertex v);

			void add_vertex(double x, double y);

			void set_vertex(unsigned n, Vertex v);

			void set_vertex(unsigned n, double x, double y);

			void set_vertices(Vertices vertices);

      virtual void set_line_style( const LineStyle& style );

      virtual Drawable::pointer start_marker();

      virtual void set_start_marker(Drawable::pointer);

      virtual Drawable::pointer end_marker();

      virtual void set_end_marker(Drawable::pointer);

      virtual Drawable::pointer vertex_marker();

      virtual void set_vertex_marker(Drawable::pointer);

      virtual void draw(Cairo::RefPtr<Cairo::Context> cairo);

      /** Draws on the provided context */
      virtual void draw_shape( Cairo::RefPtr<Cairo::Context> cairo );

      sigc::signal<void, unsigned>& signal_vertex();
      sigc::signal<void>& signal_vertices();
      sigc::signal<void> signal_start_marker();
      sigc::signal<void> signal_end_marker();
      sigc::signal<void> signal_vertex_marker();

      PAPYRUS_CLASS_NAME("Polyline");

      PAPYRUS_CLONE_METHOD( Polyline );

    protected:
			Vertices m_vertices;
      Drawable::pointer m_start_marker, m_end_marker, m_vertex_marker;
			Region m_vertex_extents;

			sigc::signal<void, unsigned> m_signal_vertex;
			sigc::signal<void> m_signal_vertices;
      sigc::signal<void> m_signal_start_marker;
      sigc::signal<void> m_signal_end_marker;
      sigc::signal<void> m_signal_vertex_marker;

			virtual void on_vertex_changed(int n);

			virtual void on_vertices_changed();

      /**
			 * Reimplemented to ensure that markers are taken into account.
	  	 */
			virtual void set_extents(const Region& extents);

  };

}

#endif
